import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
/**
 * Key - A black or white piano key that contains a reference to a sound file, 
 *       and a keyboard button to be pressed which will activate the sound file 
 *       and a simple animation. Mouse control is also possible.
 * 
 * Author - Michael Kolling, for "Introduction to Programming with Greenfoot"
 * 
 * Modified by - Matt Milan
 *
 * Date - 3/6/2012, version 2.0
 */

public class Key extends Actor
{
    private final int MAX_PARTIC_CD = 50;
    private int blankParticleCountdown = MAX_PARTIC_CD;
    private boolean isBlack = false;
    private boolean isDown = false;
    private boolean mouseBreak = false;
    private String key;
    private String sound;
    private GreenfootImage image1;
    private GreenfootImage image2;

    private Visualizer screen;
    private Color shapeColor;

    /**
     * Create a new key.
     */
    public Key(String keyName, String soundFile, boolean keyColor, Visualizer scr)
    {

        screen = scr;
        key = keyName;
        sound = soundFile;
        isBlack = keyColor;
        if (isBlack)
        {
            image1 = new GreenfootImage("black-key-down.png");
            image2 = new GreenfootImage("black-key.png");
        }
        else
        {
            image1 = new GreenfootImage("white-key-down.png");
            image2 = new GreenfootImage("white-key.png");
        }
        setImage(image2);

    }

    /**
     * Do the action for this key.
     */
    public void act()
    {
        keyBoardControl();
        mouseControl();
    }

    /**
     * play() - plays the clip referenced in the sound variable
     */
    public void play()
    {

        Greenfoot.playSound(sound);
        if (screen != null)
        {
            screen.visualize(this);
            screen.moreRain();
        }
    }

    /**
     * mouseIsWithin() - checks if current mouse position is "inside" the dimensions of
     *                   the image used to represnent the key. Returns true if so.
     * 
     */ 
    public boolean mouseIsWithin()
    {
        if (Greenfoot.getMouseInfo().getX() < getX() + (getImage().getWidth() / 2)
        && Greenfoot.getMouseInfo().getX() > getX() - (getImage().getWidth() / 2)
        && Greenfoot.getMouseInfo().getY() < getY() + (getImage().getHeight() / 2)
        && Greenfoot.getMouseInfo().getY() > getY() - (getImage().getHeight() / 2))
        {
            return true;
        }
        else 
        {
            return false;
        }
    }

    /**
     * keyBoardControl - method that allows piano to be played with the keyboard
     *                   note: even if disabled, octave can still be switched 
     *                   by keyboard.
     */
    public void keyBoardControl()
    {
        if ( !isDown && Greenfoot.isKeyDown(key) )
        {
            setImage (image1);
            play();
            isDown = true;
        }
        else if (isDown && !Greenfoot.isKeyDown(key) )
        {
            setImage (image2);
            isDown = false;
        }
    }

    /**
     * mouseControl() - enables mouse control of piano. allows for dragging across
     *                  keys, regardless of whether  mouse started on the world or
     *                  on a specific key.
     */

    public void mouseControl()
    {
        if (Greenfoot.mouseDragged(null) && mouseIsWithin() && !mouseBreak 
        || Greenfoot.mousePressed(this))
        {
            setImage(image1);
            play();
            mouseBreak = true;
        }
        if ((Greenfoot.mouseDragged(null) && mouseBreak && !mouseIsWithin()))
        {
            setImage(image2);
            mouseBreak = false;
        }        
        if (Greenfoot.mouseMoved(null) || Greenfoot.mouseClicked(null))
        {
            setImage(image2);
        }        
    }


    public void setVisualizer(Visualizer vis)
    {
        screen = vis;
    }

    public void setMyColor(Color clr)
    {
        shapeColor = clr;
    }

    public Color getMyColor()
    {
        return shapeColor;
    }

    public Visualizer getVisualizer()
    {
        return screen;
    }
    public Color returnColor()
    {
        return shapeColor;
    }
}
